/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/

#ifndef IDIOSKOPOSWRITEONLYPROPERTYBASE_H
#define IDIOSKOPOSWRITEONLYPROPERTYBASE_H

#include <idioskopos/object.h>

namespace Idioskopos
  {

  /**
   * Implements a simple write only property. Using this class ensures
   * that the value may not be read through introspection methods.
   *
   * @author Rick L Vinyard Jr
   */
  template <typename T>
  class WriteOnlyPropertyBase: public virtual Object
    {
    public:
      typedef IdioskoposPointer<WriteOnlyPropertyBase> pointer;
      typedef T value_type;

      /** Default constructor that doesn't do much. */
      WriteOnlyPropertyBase ()
      { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name.
       */
      WriteOnlyPropertyBase(Introspectable& container, const std::string& name): Object(container, name)
      { }

      /** Destructor */
      virtual ~WriteOnlyPropertyBase()
      { }

      /** Allows assignment to the property from the contained value type. */
      WriteOnlyPropertyBase<T>& operator= (const T& val)
      {
        this->set(val);
        return *this;
      }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set(const T& val) = 0;

    };

}

#endif

